package com.safeway.epe;

import com.google.common.collect.ImmutableList;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;


import java.util.Arrays;
import java.util.List;

public class CustomObjectsRDD {
    public static void main(String[] args) {
        SparkConf sparkConf = new SparkConf().setAppName("Print Elements of RDD")
                .setMaster("local[*]").set("spark.executor.memory", "2g");
        // start a spark context
        JavaSparkContext sc = new JavaSparkContext(sparkConf);

        // prepare list of objects
        List<Promotion> personList = ImmutableList.of(
                new Promotion(123,"big billion"),
                new Promotion(124,"Special Offers"),
                new Promotion(125,"bank Offers"),
                new Promotion(126,"Exrta discount"),
                new Promotion(127,"BOGO")
                );

        List<Offers> offersList =
                ImmutableList.of(new Offers(123456,"Diwali Offers",personList));

        // parallelize the list using SparkContext
        JavaRDD<Promotion> perJavaRDD = sc.parallelize(personList);
        JavaRDD<Offers> offersJavaRDD = sc.parallelize(offersList);
        JavaRDD<String> mapJavaRDD = perJavaRDD.map(promo -> promo.getPromoType().toUpperCase());
        JavaRDD<String> flatMapJavaRDD = offersJavaRDD.flatMap(offers -> Arrays.asList(offers.getPromotion().get(0).getPromoType()).iterator());

        for (Promotion promotion : perJavaRDD.collect()) {
            System.out.println(promotion.getPromoId() + " \t "+promotion.getPromoType());
        }
        for(String promo : mapJavaRDD.collect())
        {
            System.out.println("toUpperCase :"+promo);
        }

        sc.close();
    }

}
