package com.safeway.epe.kafkastreaming;
import com.safeway.epe.model.Promotion;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.spark.SparkConf;
import org.apache.spark.streaming.Durations;
import org.apache.spark.streaming.api.java.JavaDStream;
import org.apache.spark.streaming.api.java.JavaInputDStream;
import org.apache.spark.streaming.api.java.JavaStreamingContext;
import org.apache.spark.streaming.kafka010.ConsumerStrategies;
import org.apache.spark.streaming.kafka010.KafkaUtils;
import org.apache.spark.streaming.kafka010.LocationStrategies;
import org.codehaus.jackson.map.ObjectMapper;
import scala.Tuple2;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;


public class KafkaStreamingExample {
    static ObjectMapper mapper = new ObjectMapper();
    public static void main(String[] args) throws InterruptedException, IOException {
        System.setProperty("hadoop.home.dir" , "c:/hadoop");
        Logger.getLogger("org.apache").setLevel(Level.WARN);
        Logger.getLogger("org.apache.spark.storage").setLevel(Level.ERROR);
        Map<String, Object> params = new HashMap<>();
        params.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG ,"localhost:9092");
        params.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG,"earliest");
        params.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
        params.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
        params.put(ConsumerConfig.GROUP_ID_CONFIG,"schema");
        SparkConf sparkConf = new SparkConf().setAppName("Read file from Kafka").setMaster("local[*]");
       JavaStreamingContext javaStreamingContext = new JavaStreamingContext(sparkConf , Durations.seconds(2));

        JavaInputDStream<ConsumerRecord<Object, Promotion>> stream = KafkaUtils
                .createDirectStream(javaStreamingContext,
                LocationStrategies.PreferConsistent(),
                ConsumerStrategies.Subscribe(Arrays.asList("promotion"), params));
        JavaDStream<Promotion> result = stream.map(promotion -> promotion.value());
        stream.mapToPair(record -> new Tuple2<>(record.key(), record.value())).print();
        result.print();
        javaStreamingContext.start();
        javaStreamingContext.awaitTermination();
    }
}
