package com.safeway.epe.kafkastreaming;

import io.confluent.kafka.schemaregistry.client.CachedSchemaRegistryClient;
import io.confluent.kafka.schemaregistry.client.SchemaMetadata;
import io.confluent.kafka.schemaregistry.client.rest.exceptions.RestClientException;
import io.confluent.kafka.serializers.KafkaAvroDeserializer;
import org.apache.avro.Schema;
import org.apache.avro.generic.GenericRecord;
import org.apache.spark.SparkConf;
import org.apache.spark.sql.RowFactory;
import org.apache.spark.sql.SQLContext;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.avro.SchemaConverters;
import org.apache.spark.sql.types.StructType;

import java.io.IOException;

public class StreamingMongoDBExample {
    public static void main(String[] args) throws IOException, RestClientException {
        SparkConf sparkConf = new SparkConf().setAppName("sending data to mongodb")
                .setMaster("local[*]")
                .set("spark.mongodb.input.uri", "mongodb://127.0.0.1/promo");
        SparkSession session = SparkSession.builder().config(sparkConf).getOrCreate();
        SQLContext sqlContext = session.sqlContext();
        CachedSchemaRegistryClient cachedSchemaRegistryClient = new CachedSchemaRegistryClient("", 128);
        SchemaMetadata latestSchemaMetadata = cachedSchemaRegistryClient.getLatestSchemaMetadata("" + "-value");
        Schema schemaRegistryClientById = cachedSchemaRegistryClient.getById(latestSchemaMetadata.getId());
        StructType dataType = (StructType) SchemaConverters.toSqlType(schemaRegistryClientById).dataType();

        session.udf().register("deserialize", (byte[] data) -> {
            CachedSchemaRegistryClient schemaRegistryClient = new CachedSchemaRegistryClient("", 128);
            KafkaAvroDeserializer kafkaAvroDeserializer = new KafkaAvroDeserializer(schemaRegistryClient);
            GenericRecord record = (GenericRecord) kafkaAvroDeserializer.deserialize("", data);
            return RowFactory.create(Long.parseLong("" + record.get("promotionId")), record.get("promotionType").toString(),
                    record.get("discount")
            );
        }, dataType);
    }
}
