package com.safeway.epe.kafkastreaming;

import com.safeway.epe.model.Promotion;
import com.safeway.epe.producer.SparkKafkaProducer;
import io.confluent.kafka.schemaregistry.client.CachedSchemaRegistryClient;
import io.confluent.kafka.schemaregistry.client.SchemaMetadata;
import io.confluent.kafka.schemaregistry.client.rest.exceptions.RestClientException;
import io.confluent.kafka.serializers.KafkaAvroDeserializer;
import org.apache.avro.Schema;
import org.apache.avro.generic.GenericRecord;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.spark.sql.*;
import org.apache.spark.sql.avro.SchemaConverters;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.sql.types.StructType;

import java.io.IOException;

import static org.apache.spark.sql.Encoders.bean;


public class StructuredStreamingExample {
    public static void main(String[] args) throws StreamingQueryException, IOException, RestClientException {
        System.setProperty("hadoop.home.dir" , "c:/hadoop");
        Logger.getLogger("org.apache").setLevel(Level.WARN);
        Logger.getLogger("org.apache.spark.storage").setLevel(Level.ERROR);
        SparkSession session = SparkSession.builder()
                .master("local[*]")
                .appName("Structure Streaming with Kafka")
                .getOrCreate();
        SQLContext sqlContext = session.sqlContext();

        CachedSchemaRegistryClient cachedSchemaRegistryClient = new CachedSchemaRegistryClient("http://localhost:8081", 128);
        SchemaMetadata latestSchemaMetadata = cachedSchemaRegistryClient.getLatestSchemaMetadata("promotion_spark" + "-value");
        Schema schemaRegistryClientById = cachedSchemaRegistryClient.getById(latestSchemaMetadata.getId());
        StructType dataType = (StructType) SchemaConverters.toSqlType(schemaRegistryClientById).dataType();

        session.udf().register("deserialize", (byte[] data) -> {
            KafkaAvroDeserializer kafkaAvroDeserializer = new KafkaAvroDeserializer(new CachedSchemaRegistryClient("http://localhost:8081", 128));
            GenericRecord promotion = (GenericRecord) kafkaAvroDeserializer.deserialize("", data);
            return RowFactory.create(promotion.get("promotionId"), promotion.get("promotionType"), promotion.get("discount"));
       }, dataType);

        Dataset<Promotion> dataset = session.readStream()
                .format("kafka")
                .option("kafka.bootstrap.servers" , "localhost:9092")
                .option("subscribe","promotion_spark")
                .load()
                .selectExpr("deserialize(value) as value")
                .select("value.*")
                .as(bean(Promotion.class));
        dataset.createOrReplaceTempView("viewresult");

        Dataset<Promotion> promotionDataset = sqlContext.sql("select * from viewresult").as(bean(Promotion.class));
        StreamingQuery checkpointLocation = promotionDataset.writeStream()
                .format("console")
                .outputMode(OutputMode.Append())
                .option("checkpointLocation", "Desktop/temp/OfferDataProcessor")
                .foreach(new SparkKafkaProducer("promo"))
                .start();

//        Dataset<Row> result = session.sql("select * from viewresult");
//        StreamingQuery query = result.writeStream()
//                .format("console")
//                .outputMode(OutputMode.Append())
//                .start();

//        Dataset result = session.sql("select * from viewresult");
//        StreamingQuery ds = result
//                .selectExpr("CAST(key AS STRING)", "CAST(value AS STRING)")
//                .writeStream()
//                .format("kafka")
//                .option("kafka.bootstrap.servers", "localhost:9092")
//                .option("subscribe","promotion")
//                .start();
        checkpointLocation.awaitTermination(120000);
    }
}
