package com.safeway.epe.producer;

import com.safeway.epe.model.Promotion;
import com.safeway.epe.model.PromotionData;
import io.confluent.kafka.serializers.AbstractKafkaAvroSerDeConfig;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.apache.spark.sql.ForeachWriter;

import java.util.Properties;
import java.util.concurrent.Future;

public class SparkKafkaProducer extends ForeachWriter<Promotion> {
    private KafkaProducer<String , PromotionData> promotionData;
    String topic;

    public SparkKafkaProducer(String topic)
    {
        this.topic=topic;
    }
    @Override
    public boolean open(long l, long l1) {
        promotionData = kafkaProducer(PromotionData.class);
        return true;
    }
    @Override
    public void process(Promotion promotion) {

        PromotionData.Builder promotionBuilder = PromotionData.newBuilder();
        promotionBuilder.setPromotionId(promotion.getPromotionId());
        promotionBuilder.setPromotionType(promotion.getPromotionType());
        promotionBuilder.setDiscount(promotion.getDiscount());

        PromotionData promotionData = promotionBuilder.build();
        sendPromotion(promotionData);
    }
    private void sendPromotion(PromotionData promotion) {
        try {
            ProducerRecord<String, PromotionData> producerRecord = new ProducerRecord<>(topic,promotion);
            Future<RecordMetadata> metadataFuture = promotionData.send(producerRecord);
        } catch (Exception e) {
          e.printStackTrace();
        }
    }

    @Override
    public void close(Throwable throwable) {
      promotionData.close();
    }

    private <T> KafkaProducer<String, T> kafkaProducer(Class<T> clazz) {
        final Properties properties = new Properties();
        properties.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, "http://127.0.0.1:9092");
        properties.put(ProducerConfig.ACKS_CONFIG, "all");
        properties.put(ProducerConfig.RETRIES_CONFIG, "0");
        properties.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringSerializer");
        properties.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, "io.confluent.kafka.serializers.KafkaAvroSerializer");
        properties.put(AbstractKafkaAvroSerDeConfig.SCHEMA_REGISTRY_URL_CONFIG, "http://127.0.0.1:8081");
        return new KafkaProducer<>(properties);
    }
}
